# -*- coding: utf-8 -*-
# moebelgruppe.py

from grafikfenster import *
from moebel import Moebel

from abc import ABC,abstractmethod # abstrakte Klasse realisieren

### ---------------------------------------------------
class MoebelGruppe(Moebel):
    """Klasse MoebelGruppe
    Realisiert Möbelgruppen nach dem Kompositummuster"""

    def __init__(self):
        """Konstruktor mit vordefinierten Parametern
        für die Position, Winkel und Farbe;
        breite und tiefe sind zu Anfang 0,
        die Gruppe immer sichtbar"""
        self.__komponenten=[]
        Moebel.__init__(self, 0, 0, 0, 0, 0, None, False)

    def FuegeHinzu(self, moebel):
        self.__komponenten.append(moebel)
        self.HoleDaten()
    
    def HoleDaten(self):
        if len(self.__komponenten)==0:
            self._aktualisiereDaten(0,0,0,0)
            return
        path=self.GibZeichenPfad()
        for moebel in self.__komponenten:
            path.AddPath(moebel.GibFigur())
        xNeu,yNeu,bNeu,tNeu=path.GetBox()
        self._aktualisiereDaten(round(xNeu),round(yNeu),round(bNeu),round(tNeu))
        
    def Entferne(self, moebel):
        if not(moebel in self.__komponenten): return
        self.__komponenten.remove(moebel)
        self.HoleDaten()
        
    def GibKindobjekt(self, position):
        """gibt für eine MoebelGruppe das Kindobjekt zur Position zurück"""
        if position<len(self.__komponenten):
            if position>=0:
                return self.__komponenten[position]
        return None
        
    def GibFigur(self):
        """Gibt hier nur einen leeren Pfad zurueck."""
        path = self.GibZeichenPfad()
        return path
        
    def BewegeHorizontal(self, weite):
        """Veraendernde Methode fuer die x-Position"""        
        for moebel in self.__komponenten:
            moebel.BewegeHorizontal(weite)
        self.HoleDaten()
    
    def BewegeVertikal(self, weite):
        """Veraendernde Methode fuer die y-Position"""
        for moebel in self.__komponenten:
            moebel.BewegeVertikal(weite)
        self.HoleDaten()

    def Drehe(self, winkel):
        """Veraendernde Methode fuer die Orientierung [Winkel]"""
        x,y=self.GibXPosition(), self.GibYPosition()
        b,t=self.GibBreite(), self.GibTiefe()
        super(MoebelGruppe,self).Drehe(winkel)
        if self.GibOrientierung()!=0:
            zentrum=x+round(b/2),y+round(t/2)        
            for moebel in self.__komponenten:
                moebel.Drehe(self.GibOrientierung(), zentrum)

    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        for moebel in self.__komponenten:
            moebel.Verberge()

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        for moebel in self.__komponenten:
            moebel.Zeige()


### ---------------------------------------------------
class MoebelApp(wx.App):
    """Test-Anwendung speziell fuer Stuhl"""
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        ##self.TestAnwendung1()
        self.TestAnwendung2()
        return True

    def TestAnwendung1(self):
        """für eine Schrankwand"""
        global moebelgruppe
        from schrank import Schrank
        global schrank1, schrank2, schrank3
        schrank1=Schrank(0, 0, 60, 37, 0, 'black', True)
        schrank2=Schrank(60, 0, 60, 37, 0, 'red', True)
        schrank3=Schrank(120, 0, 60, 37, 0, 'green', True)
        moebelgruppe=MoebelGruppe()
        moebelgruppe.FuegeHinzu(schrank1)
        moebelgruppe.FuegeHinzu(schrank2)
        moebelgruppe.FuegeHinzu(schrank3)
        moebelgruppe.BewegeHorizontal(50)
        moebelgruppe.BewegeVertikal(30)
        moebelgruppe.Drehe(10)
        
    def TestAnwendung2(self):
        """Allein fuer die Testanwendung:
        sollte nicht gehen!"""
        global moebelgruppe
        from stuhl import Stuhl
        from tisch import Tisch
        global stuhl1, stuhl2, stuhl3, stuhl4, tisch 
        stuhl1=Stuhl(0, 60, winkel=270,sichtbar=True)
        stuhl2=Stuhl(60, 0, winkel=0,sichtbar=True)
        stuhl3=Stuhl(120, 0, winkel=0,sichtbar=True)
        stuhl4=Stuhl(180, 60, winkel=90,sichtbar=True)
        stuhl5=Stuhl(120, 120, winkel=180,sichtbar=True)
        stuhl6=Stuhl(60, 120, winkel=180,sichtbar=True)
        tisch=Tisch(50, 50, 120, 60,sichtbar=True)
        moebelgruppe=MoebelGruppe()
        moebelgruppe.FuegeHinzu(stuhl1)
        moebelgruppe.FuegeHinzu(stuhl2)
        moebelgruppe.FuegeHinzu(stuhl3)
        moebelgruppe.FuegeHinzu(stuhl4)
        moebelgruppe.FuegeHinzu(stuhl5)
        moebelgruppe.FuegeHinzu(stuhl6)
        moebelgruppe.FuegeHinzu(tisch)
##        print(moebelgruppe.GibXPosition(),
##              moebelgruppe.GibYPosition(),
##              moebelgruppe.GibBreite(),
##              moebelgruppe.GibTiefe())
        moebelgruppe.BewegeHorizontal(80)
##        print(moebelgruppe.GibXPosition(),
##              moebelgruppe.GibYPosition(),
##              moebelgruppe.GibBreite(),
##              moebelgruppe.GibTiefe())
        moebelgruppe.BewegeVertikal(30)
##        print(moebelgruppe.GibXPosition(),
##              moebelgruppe.GibYPosition(),
##              moebelgruppe.GibBreite(),
##              moebelgruppe.GibTiefe())
        moebelgruppe.Drehe(10)
        
        
### ---------------------------------------------------
if __name__ == '__main__':
    app = MoebelApp(redirect=True)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
